@@ -1,5 +1,5 @@
 {-
-    Copyright 2014-2015 Mario Blazevic
+    Copyright 2014-2018 Mario Blazevic
 
     License: BSD3 (see BSD3-LICENSE.txt file)
 -}
@@ -29,7 +29,8 @@ import Control.Applicative -- (Applicative(..))
 import qualified Data.List as List
 import Data.String (IsString(..))
 
-import Data.Monoid -- (Monoid(..), (<>), Endo(..))
+import Data.Semigroup (Semigroup(..))
+import Data.Monoid (Monoid(..), Endo(..))
 import Data.Monoid.Cancellative (LeftReductiveMonoid(..), RightReductiveMonoid(..), LeftGCDMonoid(..), RightGCDMonoid(..))
 import Data.Monoid.Null (MonoidNull(null), PositiveMonoid)
 import Data.Monoid.Factorial (FactorialMonoid(..), StableFactorialMonoid)
@@ -99,16 +100,13 @@ instance Show m => Show (LinePositioned m) where
    showsPrec prec (LinePositioned pos l lpos c) =
       ("Line " ++) . shows l . (", column " ++) . shows (pos - lpos) . (": " ++) . showsPrec prec c
 
-instance StableFactorialMonoid m => Monoid (OffsetPositioned m) where
-   mempty = pure mempty
-   mappend (OffsetPositioned p1 c1) (OffsetPositioned p2 c2) =
+instance StableFactorialMonoid m => Semigroup (OffsetPositioned m) where
+   OffsetPositioned p1 c1 <> OffsetPositioned p2 c2 =
       OffsetPositioned (if p1 /= 0 || p2 == 0 then p1 else max 0 $ p2 - length c1) (mappend c1 c2)
-   {-# INLINE mempty #-}
-   {-# INLINE mappend #-}
+   {-# INLINE (<>) #-}
 
-instance (StableFactorialMonoid m, TextualMonoid m) => Monoid (LinePositioned m) where
-   mempty = pure mempty
-   mappend (LinePositioned p1 l1 lp1 c1) (LinePositioned p2 l2 lp2 c2)
+instance (StableFactorialMonoid m, TextualMonoid m) => Semigroup (LinePositioned m) where
+   LinePositioned p1 l1 lp1 c1 <> LinePositioned p2 l2 lp2 c2
      | p1 /= 0 || p2 == 0 = LinePositioned p1 l1 lp1 c
      | otherwise = LinePositioned p2' l2' lp2' c
      where c = mappend c1 c2
@@ -118,6 +116,17 @@ instance (StableFactorialMonoid m, TextualMonoid m) => Monoid (LinePositioned m)
            countLines :: Int -> Char -> Int
            countLines n '\n' = succ n
            countLines n _ = n
+   {-# INLINE (<>) #-}
+
+instance StableFactorialMonoid m => Monoid (OffsetPositioned m) where
+   mempty = pure mempty
+   mappend = (<>)
+   {-# INLINE mempty #-}
+   {-# INLINE mappend #-}
+
+instance (StableFactorialMonoid m, TextualMonoid m) => Monoid (LinePositioned m) where
+   mempty = pure mempty
+   mappend = (<>)
    {-# INLINE mempty #-}
    {-# INLINE mappend #-}
 
@@ -227,7 +236,7 @@ instance StableFactorialMonoid m => FactorialMonoid (OffsetPositioned m) where
    length (OffsetPositioned _ c) = length c
    foldMap f (OffsetPositioned p c) = appEndo (Factorial.foldMap f' c) (const mempty) p
       where -- f' :: m -> Endo (Int -> m)
-            f' prime = Endo (\cont pos-> f (OffsetPositioned pos prime) <> cont (succ pos))
+            f' prime = Endo (\cont pos-> f (OffsetPositioned pos prime) `mappend` cont (succ pos))
 
    spanMaybe s0 f (OffsetPositioned p0 t) = rewrap $ Factorial.spanMaybe (s0, p0) f' t
       where f' (s, p) prime = do s' <- f s (OffsetPositioned p prime)
@@ -296,9 +305,10 @@ instance (StableFactorialMonoid m, TextualMonoid m) => FactorialMonoid (LinePosi
    foldMap f (LinePositioned p0 l0 lp0 c) = appEndo (Factorial.foldMap f' c) (const mempty) p0 l0 lp0
       where -- f' :: m -> Endo (Int -> Int -> Int -> m)
             f' prime = Endo (\cont p l lp-> f (LinePositioned p l lp prime)
-                                            <> if characterPrefix prime == Just '\n'
-                                               then cont (succ p) (succ l) p
-                                               else cont (succ p) l lp)
+                                            `mappend`
+                                            if characterPrefix prime == Just '\n'
+                                            then cont (succ p) (succ l) p
+                                            else cont (succ p) l lp)
 
    spanMaybe s0 f (LinePositioned p0 l0 lp0 c) = rewrap $ Factorial.spanMaybe (s0, p0, l0, lp0) f' c
       where f' (s, p, l, lp) prime = do s' <- f s (LinePositioned p l lp prime)
